from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic

class MenuManager(models.Manager):

    def get_submenu_list(self, parent, level=-1):
        """Returns the submenus of parent recursively up to level levels as a
        list"""
        children = parent.get_children()
        childrenA = []
        for child in children:
            childrenA.append([child.title, child.get_absolute_url()])

            if level is not 0 and child.get_children().count():
                childrenA[-1].append(Menu.objects.get_submenu_list(child,
                    level-1))
        return childrenA

TARGET_LIST = (
    ('_self', _("Self")),
    ('_blank', _("New Blank"))
)
class Menu(models.Model):
    title = models.CharField(max_length=255, verbose_name=_("Title"), 
        help_text=_("The text shown in the menu"))
    parent = models.ForeignKey('self', null=True, blank=True, default=0, 
        verbose_name=_("Parent menu"))
    content_type = models.ForeignKey(ContentType, 
        verbose_name=_("Content Type"), 
        help_text=_("The type of the content you would like to link to"))
    object_id = models.PositiveIntegerField(verbose_name=_("Object"),
        help_text=_("The object you would like to link to"))
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    order = models.PositiveIntegerField(default=1, blank=True,
        help_text=_("An integer specifying the ordering of the menu item."))
    target = models.CharField(max_length=10, choices=TARGET_LIST,
        default='_self', blank=True,
        verbose_name=_("Link target window"), 
        help_text=_("The window where the link should open"))
    requires_logged_in = models.BooleanField(default=False, blank=True,
        verbose_name=_("Requires log-in"))
    requires_staff = models.BooleanField(default=False, blank=True,
        verbose_name=_("Requires staff user"))
    requires_admin = models.BooleanField(default=False, blank=True,
        verbose_name=_("Requires admin user"))

    objects = MenuManager()

    class Meta:
        verbose_name = _("Menu item")
        verbose_name_plural = _("Menu items")
        ordering = ['order', 'title']
    
    def __unicode__(self):
        return u"%s (%s)" % (self.title, self.parent)
    
    def save(self, force_insert=False, force_update=False):
        super(Menu, self).save(force_insert, force_update)
    
    def get_absolute_url(self):
        return self.content_object.get_absolute_url()

    def get_children(self):
        """Returns the item's submenus"""
        return Menu.objects.filter(parent=self)


class TextMenuItem(models.Model):
    """A simple text/url menu"""
    title = models.CharField(max_length=255, verbose_name=_("Title"), 
        help_text=_("A descriptive name for this link"))
    url = models.CharField(max_length=255, verbose_name=_("URL"),
        help_text=_("The link's destination url"))
    
    class Meta:
        verbose_name = _("Text menu item")
        verbose_name_plural = _("Text menu items")
    
    def __unicode__(self):
        return u'%s (%s)' % (self.title, self.url)
    
    def save(self, force_insert=False, force_update=False):
        super(TextMenuItem, self).save(force_insert, force_update)
    
    def get_absolute_url(self):
        return self.url

