from django.test import TestCase
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
import models, forms

class MenuTest(TestCase):

    def setUp(self):
        self.example_item = models.TextMenuItem.objects.create(title="Test item",
            url="http://djangoproject.com")

    def test_with_defaults(self):
        '''
        Tests a correct instance with default values
        '''
        menu = models.Menu.objects.create(title='Test menu',
            content_object=self.example_item)
        self.assertTrue(menu.pk)

    def test_simple(self):
        '''
        Tests a correct instance with all the values
        '''
        menu = models.Menu.objects.create(title="Test menu", parent=None,
            content_object=self.example_item)
        self.assertTrue(menu.pk)
        return menu

    def test_absolute_url(self):
        '''
        Tests that the correct url is returned
        '''
        menu = self.test_simple()
        self.assertEqual(menu.get_absolute_url(), "http://djangoproject.com")

    def test_get_children(self):
        '''
        Tests that child elements are returned fine
        '''
        menu = self.test_simple()
        submenu = models.Menu.objects.create(title="Submenu",
            parent=menu, content_object=self.example_item)
        notin = models.Menu.objects.create(title="Submenu",
            parent=None, content_object=self.example_item)
        self.assertEqual(menu.get_children().count(), 1)
        self.assertEqual(menu.get_children()[0], submenu)

class MenuManagerTest(TestCase):
    '''
    Tests a multi-level menu
    '''

    def setUp(self):
        """Create a menu structure"""
        item = models.TextMenuItem.objects.create(title="Test title",
            url="http://example.com")
        data = {
            'title': 'MainMenu',
            'parent': None,
            'content_object': item,
        }
        m1 = models.Menu.objects.create(**data.copy())
        data.update({'title': 'Base',
            'parent': m1})
        m2 = models.Menu.objects.create(**data.copy())
        data.update({'title': 'Sibling',
            'parent': m1})
        m3 = models.Menu.objects.create(**data.copy())
        data.update({'title': 'Child1',
            'parent': m2})
        m4 = models.Menu.objects.create(**data.copy())
        data.update({'title': 'Child2',
            'parent': m2})
        m5 = models.Menu.objects.create(**data.copy())
        data.update({'title': 'SubChild1',
            'parent': m4})
        m6 = models.Menu.objects.create(**data.copy())

    def test_full_list(self):
        self.assertEqual(
            models.Menu.objects.get_submenu_list(models.Menu.objects.get(pk=1)),
            [[u'Base', u'http://example.com', [
                [u'Child1', u'http://example.com', [
                    [u'SubChild1', u'http://example.com']
                    ]],
                [u'Child2', u'http://example.com']
                ]],
             [u'Sibling', u'http://example.com']]
            )

    def test_level(self):
        self.assertEqual(
            models.Menu.objects.get_submenu_list(
                models.Menu.objects.get(pk=2), 0),
            [[u'Child1', u'http://example.com',],
             [u'Child2', u'http://example.com']]
            )
        self.assertEqual(
            models.Menu.objects.get_submenu_list(
                models.Menu.objects.get(pk=2), 1),
            [[u'Child1', u'http://example.com', [
                [u'SubChild1', u'http://example.com']
                ]],
             [u'Child2', u'http://example.com']]
            )


class TextMenuItemTest(TestCase):

    def test_simple(self):
        '''
        Tests a well-specified instance
        '''
        item = models.TextMenuItem.objects.create(title="Test title",
            url="http://example.com")
        self.assertTrue(item.pk)
        self.assertEqual(item.get_absolute_url(), 'http://example.com')

class MenuAdminFormTest(TestCase):

    def setUp(self):
        self.item = models.TextMenuItem.objects.create(title="Test title",
            url="http://example.com")

    def test_basic(self):
        """Tests that the menu gets saved properly"""
        data = {
            'title': 'My Menu',
            'parent': None,
            'order': 1,
            'content_type': ContentType.objects.get_for_model(self.item).pk,
            'object_id': self.item.pk,
        }
        form = forms.MenuAdminForm(data)
        self.assertTrue(form.is_valid())
        menu = form.save()
        self.assertTrue(isinstance(menu, models.Menu))
        self.assertEqual(menu.content_object, self.item)

class GetObjectsView(TestCase):

    def setUp(self):
        User.objects.create_user(username='user', password='user',
            email='user@example.com')
        self.item = models.TextMenuItem.objects.create(title="Test",
            url="http://example.com")
    
    def test_login(self):
        rsp = self.client.get(reverse('vmenu_admin_update_list'), {'content_type':
            ContentType.objects.get_for_model(self.item).pk})
        self.assertEqual(rsp.status_code, 302)

    def test_basic(self):
        self.client.login(username='user', password='user')
        rsp = self.client.get(reverse('vmenu_admin_update_list'), {'content_type':
            ContentType.objects.get_for_model(self.item).pk})
        self.assertEqual(rsp.status_code, 200)
        self.assertEqual(rsp.content, '''<option value="">Choose</option>
 
<option value="1">Test (http://example.com)</option>

''')


class GetURLView(TestCase):

    def setUp(self):
        User.objects.create_user(username='user', password='user',
            email='user@example.com')
        self.item = models.TextMenuItem.objects.create(title="Test",
            url="http://example.com")
    
    def test_login(self):
        rsp = self.client.get(reverse('vmenu_admin_url'), {'content_type':
            ContentType.objects.get_for_model(self.item).pk,
            'object_id': 1})
        self.assertEqual(rsp.status_code, 302)

    def test_basic(self):
        self.client.login(username='user', password='user')
        rsp = self.client.get(reverse('vmenu_admin_url'), {'content_type':
            ContentType.objects.get_for_model(self.item).pk,
            'object_id': 1})
        self.assertEqual(rsp.status_code, 200)
        self.assertEqual(rsp.content, 'http://example.com')


# write templatetag tests
# write tests for login_required, staff, etc too
